/** \file main.c
 *
 * \brief Timer interrupt.
 *
 * This example program switches LEDs on and off using a timer interrupt.  
 *
 * The relevant registers for the are:
 * - TCNTx: Timer/Counter Value Register
 * - OCRxA: Output Compare Register A
 * - TCCRxB: Timer/Counter Control Register B
 *   - CSx0: Clock Select
 *   - WGMx2: Waveform Generation Mode 
 * - TIMSKx: Timer/Counter Interrupt Mask Register
 *   - TOIEx: Overflow Interrupt Enable
 *   - OCIExA: Output Compare A Match Interrupt Enable 
 *
 * \note
 * Timers can run in different modes of operation. In normal mode the
 * timer counts up to MAX (0xFFFF) and throws a timer overflow interrupt.
 * In clear timer on compare catch (CTC) mode the timer counts up to TOP
 * (OCRxA) and throws a compare match interrupt. In order to control the
 * period of the timer the either starting value (BOTTOM) or the top value
 * (TOP) has to be set. In normal mode BOTTOM (TCNT) is set, in CTC mode
 * TOP (OCRxA).
 *
 * For normal mode BOTTOM is calculate by TCNTx = 65536 - F_CPU/(n*TPS)
 * For CTC mode TOP is calculate by OCRxA = F_CPU/(n*TPS) - 1
 * - n: Prescale value
 * - TPS: Ticks per seconds
 * - F_CPU: System frequency
 *
 * Please see modules 4.3.2 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.2
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-11-27 \n
 *      Modified: 2018-04-29 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

// This example demonstrates nomal mode and CTC mode. Therefore we use tow timers.
// Timer TC1 runs in normal mode, timer TC3 runs in CTC mode.  We want LED0 and LED1
// to blink with a frequency of 1 Hz. This means that half of the time the LEDs are
// on and half of the time they are off. Therefore we need 2 interrupts per second
// to toggle the LEDs. We use a prescaler of 256. So we have to set TCNT1 (BOTTOM)
// to 43936 (n=256, TPS=2, F_CPU=11059200) and OCR3A (TOP) to 21599. The job is
// entirely done in the interrupt service routine. Therefore the superloop has
// nothing to do.
int main(void)
{
	/***** Initialize LEDs *****/
	LED_DDR = (1 << LED0) | (1 << LED1);
	LED_PORT = (1 << LED0) | (1 << LED1);

	/***** Initialize TC1 to run in normal mode. *****/
    // Set BOTTOM value.
	TCNT1 = 43937;
    // Enable timer overflow interrupt.
	TIMSK1 = (1 << TOIE1);
	// Enable clock by setting the prescaler to 256.
	TCCR1B = (4 << CS10);

	/***** Initialize TC3 to run in CTC mode. *****/
    // Set TOP value.
	OCR3A = 21599;
	// Enable compare match interrupt.
	TIMSK3 = (1 << OCIE3A);
	// Select CTC mode and enable clock by setting the prescaler to 256.
	TCCR3B = (1 << WGM32) | (4 << CS30);

	// Enable Interrupts 
	sei();

	while(1)
	{
		// Nothing to do
	}
}

/***** Timer/counter overflow interrupt *****/
// This interrupt service routine gets called two times per second.
ISR(TIMER1_OVF_vect)
{
	// Toggle the LED.
	LED_PORT ^= (1 << LED0);
	// Set the BOTTOM value again.
	TCNT1 = 43937;
}

/***** Compare match interrupt *****/
// This interrupt service routine gets called two times per second.
ISR(TIMER3_COMPA_vect)
{
	// Toggle the LED.
	LED_PORT ^= (1 << LED1);
}
