/** \file main.c
 *
 * \brief USART interrupt.
 *
 * This example program receives characters from the console and sends them back.
 *
 * The relevant registers are:
 * - UCSRxB: USART Control and Status Register B
 *   - RXCIEx:?RX Complete Interrupt Enable
 *
 * \note
 * This program implements example '04. USART' of the peripherals course with the
 * aid of a receive complete interrupt. Only the interrupt part is explained here.
 * Please see the peripherals course for more information on the USART part.
 *
 * \warning
 * This program demonstrates the use of the receive complete interrupt.
 * It is not correct as there is a race condition. Race conditions are
 * discussed in the next module and not taken into account here. Please
 * see the advanced example for a correct version of the program.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-11-27 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

#define BUFFER_FULL 1
#define BUFFER_EMPTY 0

// Variables which are changed by interrupts must be declared as volatile.
volatile uint8_t buffer;
volatile uint8_t status = BUFFER_EMPTY;

// We want to send back the characters which we receive via the USART interface.
// We use the receive complete interrupt to handle the reception and store the
// character received in a buffer. If the buffer is full we send it back in
// the super loop.
int main(void)
{
    // Initialize the USART pins.
	USB_USART_PIN |= (1 << USB_USART_TX);
	USB_USART_DDR |= (1 << USB_USART_TX);
	USB_USART_DDR &= ~(1 << USB_USART_RX);

    // Initialize the USART and enable RXC interrupt.
	USB_USART_UBRR = USB_USART_BAUDRATE;
	USB_USART_UCSRC = USB_USART_CONFIG;
	USB_USART_UCSRB= (1 << RXCIE1) |(1 << RXEN1) | (1 << TXEN1);

	// Enable interrupts.
	sei();

	while(1)
	{
		// Echo data if buffer is full.
		if (status == BUFFER_FULL) {
			// Wait until data register is empty.
			while(!(USB_USART_UCSRA & (1 << UDRE1)));  // Wait until data register empty
			// Send character stored in buffer.
			USB_USART_UDR = buffer;                    // Send data
			// The buffer is now empty again.
            status = BUFFER_EMPTY;
		}
	}
}

/*****  USART receive complete interrupt *****/
// This interrupt service routine gets called when new data has been received.
ISR(USB_USART_RXC_vect)
{
	// Store character in buffer.
	buffer = USB_USART_UDR;
	// Say that buffer is full.
	status = BUFFER_FULL;
}
