/** \file main.c
 *
 * \brief USART interrupt - advanced version.
 *
 * This version of '02 USART' is advanced training material for experienced students.
 *
 * Race conditions occur if data access is interrupted. In order to prevent race
 * conditions data access must be atomic. This means that it must not be interrupted.
 * One way to make data access atomic is to disable the interrupts. Disabling interrupts
 * is a bad idea. They should be disabled for a very short, only. This example shows
 * how this can be done.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-21
 *
 * \par History
 *      Created: 2018-07-21
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

#define BUFFER_FULL 1
#define BUFFER_EMPTY 0

// Compilers are allowed to optimize code. They suppose that variables
// can never change unexpectedly. This is not true if variables are
// modified by interrupts. Therefore such variables have to be
// declared volatile.
volatile uint8_t buffer;
volatile uint8_t status = BUFFER_EMPTY;

// We want to send back the characters which we receive via the USART interface.
// We use the receive complete interrupt to handle the reception and store the
// character received in a buffer. If the buffer is full we send it back in
// the super loop.
int main(void)
{
    // Initialize the USART pins.
	USB_USART_PIN |= (1 << USB_USART_TX);
	USB_USART_DDR |= (1 << USB_USART_TX);
	USB_USART_DDR &= ~(1 << USB_USART_RX);

    // Initialize the USART and enable RXC interrupt.
	USB_USART_UBRR = USB_USART_BAUDRATE;
	USB_USART_UCSRC = USB_USART_CONFIG;
	USB_USART_UCSRB= (1 << RXCIE1) |(1 << RXEN1) | (1 << TXEN1);

	// Enable interrupts.
	sei();

	while(1)
	{
		// Echo data if buffer is full.
		if (status == BUFFER_FULL) {
			/***** Atomic section starts here. *****/
			cli(); // Disable interrupts.
			// The period where the interrupts are disabled must be as short as possible.
			// We can not wait until the data register of the USART is empty (DRE).
			// Therefore we store the content of the buffer in a temporary variable.
			uint8_t temp = buffer;
			// We saved the content. The buffer is free again and can get new data.
			status = BUFFER_EMPTY;
			sei(); // Enable interrupts
			/***** Atomic section ends here. *****/
			// Interrupts are enabled again. Now we have time to wait.
			while(!(USB_USART_UCSRA & (1 << UDRE1)));
			// Send character stored in temporary buffer.
			USB_USART_UDR = temp;
		}
	}
}

/*****  USART receive complete interrupt *****/
// This interrupt service routine gets called when new data has been received.
ISR(USB_USART_RXC_vect)
{
	// Store character in buffer.
	buffer = USB_USART_UDR;
	// Say that buffer is full.
	status = BUFFER_FULL;
}
