/** \file main.c
 *
 * \brief Port interrupt.
 *
 * This example program checks if push-buttons are pressed.
 *
 * The relevant registers are:
 * - EICRA: External Interrupt Control Register A
 *   - ISCn0: Interrupt Sense Control n
 * - EIMSK: External Interrupt Mask Register
 *   - INTn: External Interrupt Request n Enable 
 *
 * \note
 * There are three external interrupts: INT0/PD2, INT1/PD3 and INT2/PB2.
 * SW2 is connected to PB2. It can therefore trigger INT2.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-11-27 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

// We want to switch LED0 on if button 1 is pressed and we want to
// switch it off if button 2 is pressed. Button 2 throws an interrupt.
int main(void)
{
	// Initialize the LED.
	LED_DDR = (1 << LED1);
	LED_PORT = (1 << LED1);

	// Initialize push-buttons
	BUTTON_PORT |= ((1 << BUTTON1) | (1 << BUTTON2));

	// Initialize the external interrupt.
	EICRA = (2 << ISC20);
	EIMSK = (1 << INT2);

	// Enable interrupts. 
	sei();

	while(1)
	{
		if (!(BUTTON_PIN & (1 << BUTTON1))) {  // Button 0 pressed?
			LED_PORT &= ~(1 << LED1);          // Switch LED on
		}
	}
}

/*****  External interrupt 2 *****/
// This interrupt service routine gets called whenever button 2 is pressed.
ISR(INT2_vect)
{
	// Switch the LED off.
    LED_PORT |= (1 << LED1);
}
